/***************************************************************************
*   Copyright (C) 2011-2017 by Denis Gottardello                          *
*   info@denisgottardello.it                                              *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 3 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "qdfileattributes.h"
#include "ui_qdfileattributes.h"

QDFileAttributes::QDFileAttributes(QWidget *parent) : QDialog(parent), ui(new Ui::QDFileAttributes) {
    ui->setupUi(this);
}

QDFileAttributes::~QDFileAttributes() {
    delete ui;
}

void QDFileAttributes::on_QDTELastAccess_dateTimeChanged(const QDateTime &date) {
    ui->QPBApply->setEnabled(true);
}

void QDFileAttributes::on_QDTELastModified_dateTimeChanged(const QDateTime &date) {
    ui->QPBApply->setEnabled(true);
}

void QDFileAttributes::on_QPBApply_clicked() {
    if (!SetFileLastAccessModified(ui->QLEFile->text(), LastAccess+ QDTLastAccess.secsTo(ui->QDTELastAccess->dateTime()), LastModified+ QDTLastModified.secsTo(ui->QDTELastModified->dateTime()))) QMessageBox::critical(this, "Error!!!", "Unable to apply settings!", "Ok");
    else ReadAttributes();
}

void QDFileAttributes::on_QPBSelectFile_clicked() {
    QFileDialog FileDialog;
    FileDialog.setViewMode(QFileDialog::Detail);
    FileDialog.setDirectory(QFileInfo(ui->QLEFile->text()).dir().absolutePath());
    if (FileDialog.exec()== QDialog::Accepted) {
        ui->QLEFile->setText(FileDialog.selectedFiles().at(0));
        ReadAttributes();
    }
}

void QDFileAttributes::on_QPBQuit_clicked() {
    this->close();
}

void QDFileAttributes::ReadAttributes() {
    struct stat FileAttributes;
    stat(QString(ui->QLEFile->text()).toStdString().c_str(), &FileAttributes);
    LastAccess= FileAttributes.st_atime;
    LastModified= FileAttributes.st_mtime;
    ui->QDTELastAccess->setDateTime(QDateTime::fromTime_t(LastAccess));
    ui->QDTELastModified->setDateTime(QDateTime::fromTime_t(LastModified));
    // Remember QDateTime.toTime_t() bug on Windows during daylight savings time
    QDTLastAccess= QDateTime::fromTime_t(LastAccess);
    QDTLastModified= QDateTime::fromTime_t(LastModified);
    ui->QPBApply->setEnabled(false);
}
