/***************************************************************************
*   Copyright (C) 2011-2017 by Denis Gottardello                          *
*   info@denisgottardello.it                                              *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 3 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "qfmainwindow.h"
#include "ui_qfmainwindow.h"

QFMainWindow::QFMainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::QFMainWindow) {
    ui->setupUi(this);
    UpdateInAction= true;
    ThAnalyzeA= ThAnalyzeB= NULL;
    ThSynchronize= NULL;
    HTMLPOpup= new QDHTMLPOpup(this);
    QAction *QAMinimize= new QAction(tr("Minimize"), this);
    QAction *QARestore= new QAction(tr("Restore"), this);
    QAction *QAQuit= new QAction(tr("Quit"), this);
    connect(QAMinimize, SIGNAL(triggered()), this, SLOT(hide()));
    connect(QARestore, SIGNAL(triggered()), this, SLOT(showNormal()));
    connect(QAQuit, SIGNAL(triggered()), this, SLOT(on_QPBQuit_clicked()));
    QMenu *QMTrayIcon= new QMenu(this);
    QMTrayIcon->addAction(QAMinimize);
    QMTrayIcon->addAction(QARestore);
    QMTrayIcon->addSeparator();
    QMTrayIcon->addAction(QAQuit);
    QSTITrayIcon= new QSystemTrayIcon(this);
    QSTITrayIcon->setToolTip(this->windowTitle());
    QSTITrayIcon->setIcon(this->windowIcon());
    QSTITrayIcon->setContextMenu(QMTrayIcon);
    QSTITrayIcon->show();
    QMenu *QMTools= new QMenu(this);
    QAction *QAFileAttributes= new QAction(tr("File attributes"), this);
    QAction *QAEmptyDirectory= new QAction(tr("Empty a directory"), this);
    connect(QAFileAttributes, SIGNAL(triggered()), this, SLOT(FileAttributes()));
    connect(QAEmptyDirectory, SIGNAL(triggered()), this, SLOT(EmptyDirectory()));
    QMTools->addAction(QAFileAttributes);
    QMTools->addAction(QAEmptyDirectory);
    ui->QTBTools->setMenu(QMTools);
    QMenu *QMAdd= new QMenu(this);
    QAction *QAAddDirectories= new QAction(tr("Add directories"), this);
    QAction *QAAddFiles= new QAction(tr("Add files"), this);
    connect(QAAddDirectories, SIGNAL(triggered()), this, SLOT(AddDirectories()));
    connect(QAAddFiles, SIGNAL(triggered()), this, SLOT(AddFiles()));
    QMAdd->addAction(QAAddDirectories);
    QMAdd->addAction(QAAddFiles);
    ui->QTBExclusionListAdd->setMenu(QMAdd);
    QSettings *QSSettings= new QSettings(QDir::homePath()+ QDir::separator()+ ".BackupTo"+ QDir::separator()+ "BackupTo.ini", QSettings::IniFormat); {
        QSSettings->beginGroup("Profiles");
        int count= 0;
        while (true) {
            Profile profile;
            profile.Description= QSSettings->value("Description"+ QString::number(count), "").toString();
            profile.DeleteFilesOnTargetSide= QSSettings->value("DeleteFilesOnTargetSide"+ QString::number(count), "true").toBool();
            profile.Direction= QSSettings->value("Direction"+ QString::number(count), "0").toInt();
            profile.IgnoreFileDate= QSSettings->value("IgnoreFileDate"+ QString::number(count), "false").toBool();
            profile.SideA= QSSettings->value("SideA"+ QString::number(count), "").toString();
            profile.SideB= QSSettings->value("SideB"+ QString::number(count), "").toString();
            profile.ShowDetails= QSSettings->value("ShowDetails"+ QString::number(count), "false").toBool();
            profile.Snapshot= QSSettings->value("Snapshot"+ QString::number(count), "false").toBool();
            int count_2= 0;
            while (true) {
                QString Path= QSSettings->value("ExclusionList"+ QString::number(count)+ "."+ QString::number(count_2), "").toString();
                if (Path.trimmed().compare("")!= 0) profile.QSLExclusionFiles.append(Path);
                else break;
                count_2++;
            }
            if (profile.Description.trimmed().compare("")== 0) break;
            else {
                ui->QLWProfiles->addItem(profile.Description);
                QVProfiles.append(profile);
                count++;
            }
        }
    }{
        delete QSSettings;
    }
    QTIcons.setInterval(100);
    connect(&QTIcons, SIGNAL(timeout()), this, SLOT(UpdateIcons()));
    LastIcon= 0;
    Mode= MODENULL;
    QMEmptyDirsA= new QMenu(this);
    QMEmptyDirsA->addAction(new QAction(tr("Select All"), this));
    QMEmptyDirsA->addAction(new QAction(tr("Select None"), this));
    QMEmptyDirsA->addAction(new QAction(tr("Delete"), this));
    connect(QMEmptyDirsA->actions().at(0), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideASelectAll_clicked()));
    connect(QMEmptyDirsA->actions().at(1), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideASelectNone_clicked()));
    connect(QMEmptyDirsA->actions().at(2), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideADelete_clicked()));
    QMEmptyDirsA->actions().at(2)->setIcon(ui->QPBEmptyDirsSideADelete->icon());
    ui->QLWEmpyDirsOnSideA->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui->QLWEmpyDirsOnSideA, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(EmptyDirsAPopup(QPoint)));
    QMEmptyDirsB= new QMenu(this);
    QMEmptyDirsB->addAction(new QAction(tr("Select All"), this));
    QMEmptyDirsB->addAction(new QAction(tr("Select None"), this));
    QMEmptyDirsB->addAction(new QAction(tr("Delete"), this));
    connect(QMEmptyDirsB->actions().at(0), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideBSelectAll_clicked()));
    connect(QMEmptyDirsB->actions().at(1), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideBSelectNone_clicked()));
    connect(QMEmptyDirsB->actions().at(2), SIGNAL(triggered()), this, SLOT(on_QPBEmptyDirsSideBDelete_clicked()));
    QMEmptyDirsB->actions().at(2)->setIcon(ui->QPBEmptyDirsSideBDelete->icon());
    ui->QLWEmpyDirsOnSideB->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui->QLWEmpyDirsOnSideB, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(EmptyDirsBPopup(QPoint)));
    UpdateInAction= false;
}

QFMainWindow::~QFMainWindow() {
    QTIcons.stop();
    if (ThAnalyzeA!= NULL) delete ThAnalyzeA;
    if (ThAnalyzeB!= NULL) delete ThAnalyzeB;
    if (ThSynchronize!= NULL) delete ThSynchronize;
    delete HTMLPOpup;
    delete ui;
}

void QFMainWindow::AddItem(QListWidget *QLWList, QString Path, QBrush Color) {
    if (QLWList->count()< 1024 * 24) {
        QLWList->addItem(Path);
        QLWList->item(QLWList->count()-1)->setForeground(Color);
    } else if (QLWList->count()== 1024 * 24) {
        QLWList->addItem("Too much files");
        QLWList->item(QLWList->count()-1)->setForeground(Qt::red);
    }
}

void QFMainWindow::AddFiles() {
    QFileDialog FileDialog;
    FileDialog.setViewMode(QFileDialog::Detail);
    FileDialog.setFileMode(QFileDialog::ExistingFiles);
    if (FileDialog.exec()== QDialog::Accepted) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles.append(FileDialog.selectedFiles());
        ui->QLWExclusionList->clear();
        ui->QLWExclusionList->addItems(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles);
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::AddDirectories() {
    QFileDialog FileDialog;
    FileDialog.setViewMode(QFileDialog::Detail);
    FileDialog.setFileMode(QFileDialog::DirectoryOnly);
    if (FileDialog.exec()== QDialog::Accepted) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles.append(FileDialog.selectedFiles());
        ui->QLWExclusionList->clear();
        ui->QLWExclusionList->addItems(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles);
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::EmptyDirectory() {
    QDEmptyDirectory EmptyDirectory(this);
    EmptyDirectory.exec();
}

void QFMainWindow::EmptyDirsAPopup(QPoint pos) {
    QMEmptyDirsA->exec(ui->QLWEmpyDirsOnSideA->mapToGlobal(pos));
}

void QFMainWindow::EmptyDirsBPopup(QPoint pos) {
    QMEmptyDirsB->exec(ui->QLWEmpyDirsOnSideB->mapToGlobal(pos));
}

void QFMainWindow::FileAttributes() {
    QDFileAttributes FileAttributes(this);
    FileAttributes.exec();
}

void QFMainWindow::GetAnalyzeComplete(int Side) {
    switch(Side) {
        case SIDEA: {
            SideAComplete= true;
            QListWidgetItem* ListWidgetItem= new QListWidgetItem(tr("Side A complete in ")+ QString::number(QDTStart.secsTo(QDateTime::currentDateTime()))+ tr(" seconds")+ ", "+ QString::number(QVSideListA.count())+ " files", ui->QLWLog);
            ListWidgetItem->setData(Qt::ForegroundRole, QColor("#0000ff"));
            ui->QLWLog->addItem(ListWidgetItem);
            break;
        }
        case SIDEB: {
            SideBComplete= true;
            QListWidgetItem* ListWidgetItem= new QListWidgetItem(tr("Side B complete in ")+ QString::number(QDTStart.secsTo(QDateTime::currentDateTime()))+ tr(" seconds")+ ", "+ QString::number(QVSideListB.count())+ " files", ui->QLWLog);
            ListWidgetItem->setData(Qt::ForegroundRole, QColor("#0000ff"));
            ui->QLWLog->addItem(ListWidgetItem);
            break;
        }
    }
}

void QFMainWindow::GetEmptyDirs(int Side, QString Path) {
    switch(Side) {
        case SIDEA: {
            ui->QLWEmpyDirsOnSideA->addItem(Path);
            break;
        }
        case SIDEB: {
            ui->QLWEmpyDirsOnSideB->addItem(Path);
            break;
        }
    }
}

void QFMainWindow::GetFiles(int Side, long Files, long Directories, QVector<FileItem> *QVSideList) {
    switch(Side) {
        case SIDEA: {
            ui->QLFilesDirectoriesA->setText(QString::number(Files)+ " files, "+ QString::number(Directories)+ " directories.");
            for (int count= 0; count< QVSideList->size(); count++) {
                QVSideListA.append(QVSideList->at(count));
                if (QVSideList->at(count).IsDirectory) AddItem(ui->QLWSideA, QVSideList->at(count).Path+ QVSideList->at(count).File, Qt::black);
                else AddItem(ui->QLWSideA, QVSideList->at(count).Path+ QVSideList->at(count).File, Qt::blue);
            }
            break;
        }
        case SIDEB: {
            ui->QLFilesDirectoriesB->setText(QString::number(Files)+ " files, "+ QString::number(Directories)+ " directories.");
            for (int count= 0; count< QVSideList->size(); count++) {
                QVSideListB.append(QVSideList->at(count));
                if (QVSideList->at(count).IsDirectory) AddItem(ui->QLWSideB, QVSideList->at(count).Path+ QVSideList->at(count).File, Qt::black);
                else AddItem(ui->QLWSideB, QVSideList->at(count).Path+ QVSideList->at(count).File, Qt::blue);
            }
            break;
        }
    }
}

void QFMainWindow::GetLog(QString Row) {
    GetLog(Row, "#0000FF");
}

void QFMainWindow::GetLog(QString Row, QString Color) {
    while (ui->QLWLog->count()> 256) ui->QLWLog->takeItem(0);
    QListWidgetItem* ListWidgetItem= new QListWidgetItem(Row, ui->QLWLog);
    ListWidgetItem->setData(Qt::ForegroundRole, QColor(Color));
    ui->QLWLog->addItem(ListWidgetItem);
    ui->QLWLog->scrollToBottom();
}

void QFMainWindow::GetLog(QString Row, QString Color, long Objects) {
    GetLog(Row, Color);
    ui->QPBTotalProgress->setValue(Objects);
}

void QFMainWindow::GetSynchronizeComplete() {
    if (QDTStart.secsTo(QDateTime::currentDateTime())> 60) {
        GetLog(tr("Synchronization complete in ")+ QString::number(QDTStart.secsTo(QDateTime::currentDateTime()) / 60)+ tr(" minutes and ")+ QString::number(QDTStart.secsTo(QDateTime::currentDateTime()) % 60)+ tr(" seconds"));
    } else {
        GetLog(tr("Synchronization complete in ")+ QString::number(QDTStart.secsTo(QDateTime::currentDateTime()))+ tr(" seconds"));
    }
    HTMLPOpup->ShowHTMLPopup("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\"><html><body style=\"overflow:hidden\" bgcolor=#0000ff><font color=\"#ffffff\">"+ tr("Synchronization complete")+ "<br></p></body></html>");
    SideAComplete= true;
}

void QFMainWindow::on_QAAuthor_triggered() {
    QMessageBox::information(this, "Info", "Denis Gottardello\nwww.denisgottardello.it\ninfo@denisgottardello.it", "Ok");
}

void QFMainWindow::on_QAQtVersion_triggered() {
    QMessageBox::information(this, "Info", tr("Qt version: ")+ QT_VERSION_STR, "Ok");
}

void QFMainWindow::on_QAVersion_triggered() {
    QMessageBox::information(this, "Info", tr("BackupTo, version ")+ ("1.0.6.0"), "Ok");
}

void QFMainWindow::on_QCBDeleteFilesOnTargetSide_toggled(bool checked) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].DeleteFilesOnTargetSide= checked;
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QCBIgnoreFileDate_toggled(bool checked) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].IgnoreFileDate= checked;
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QCBShowDetails_toggled(bool checked) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].ShowDetails= checked;
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QCBSnapshot_toggled(bool checked) {
    if (!UpdateInAction) {
        UpdateInAction= true;
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Snapshot= checked;
        if (checked) ui->QRBSideAToSideB->click();
        ui->QPBSave->setEnabled(true);
        UpdateInAction= false;
    }
}

void QFMainWindow::on_QLEDescription_textChanged(const QString &) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Description= ui->QLEDescription->text();
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QLESideA_textChanged(const QString &) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA= ui->QLESideA->text();
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QLESideB_textChanged(const QString &) {
   if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB= ui->QLESideB->text();
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QLWExclusionList_currentRowChanged(int currentRow) {
    ui->QPBExclusionListRemove->setEnabled(false);
    if (currentRow> -1 && currentRow< QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles.size()) ui->QPBExclusionListRemove->setEnabled(true);
}

void QFMainWindow::on_QLWProfiles_clicked(const QModelIndex &index) {
    on_QLWProfiles_currentRowChanged(index.row());
}

void QFMainWindow::on_QLWProfiles_currentRowChanged(int currentRow) {
    if (currentRow> -1 && currentRow< ui->QLWProfiles->count()) {
        UpdateInAction= true;
        ui->QLEDescription->setText(QVProfiles[currentRow].Description);
        ui->QCBDeleteFilesOnTargetSide->setChecked(QVProfiles[currentRow].DeleteFilesOnTargetSide);
        ui->QCBIgnoreFileDate->setChecked(QVProfiles[currentRow].IgnoreFileDate);
        ui->QCBShowDetails->setChecked(QVProfiles[currentRow].ShowDetails);
        ui->QCBSnapshot->setChecked(QVProfiles[currentRow].Snapshot);
        ui->QLESideA->setText(QVProfiles[currentRow].SideA);
        ui->QLESideB->setText(QVProfiles[currentRow].SideB);
        switch (QVProfiles[currentRow].Direction) {
            case SIDEAEQUALTOSDIDEB: {
                ui->QRBSideAEqualToSideB->setChecked(true);
                break;
            }
            case SIDEATOSIDEB: {
                ui->QRBSideAToSideB->setChecked(true);
                break;
            }
            case SIDEBTOSDIDEA: {
                ui->QRBSideBToSideA->setChecked(true);
                break;
            }
        }
        ui->QLWExclusionList->clear();
        ui->QLWExclusionList->addItems(QVProfiles[currentRow].QSLExclusionFiles);
        ui->QLEDescription->setEnabled(true);
        ui->QCBDeleteFilesOnTargetSide->setEnabled(true);
        ui->QCBIgnoreFileDate->setEnabled(true);
        ui->QCBShowDetails->setEnabled(true);
        ui->QCBSnapshot->setEnabled(true);
        ui->QLESideA->setEnabled(true);
        ui->QLESideB->setEnabled(true);
        ui->QLWExclusionList->setEnabled(true);
        ui->QPBAnalyze->setEnabled(true);
        ui->QTBExclusionListAdd->setEnabled(true);
        ui->QPBRemoveProfile->setEnabled(true);
        ui->QPBSynchronize->setEnabled(false);
        ui->QRBSideAEqualToSideB->setEnabled(true);
        ui->QRBSideAToSideB->setEnabled(true);
        ui->QRBSideBToSideA->setEnabled(true);
        ui->QTBSideA->setEnabled(true);
        ui->QTBSideB->setEnabled(true);
        UpdateInAction= false;
    }
}

void QFMainWindow::on_QPBAddProfile_clicked() {
    ui->QLEDescription->setEnabled(false);
    ui->QCBDeleteFilesOnTargetSide->setEnabled(false);
    ui->QCBIgnoreFileDate->setEnabled(false);
    ui->QCBShowDetails->setEnabled(false);
    ui->QCBSnapshot->setEnabled(false);
    ui->QLESideA->setEnabled(false);
    ui->QLESideB->setEnabled(false);
    ui->QLWExclusionList->setEnabled(false);
    ui->QTBExclusionListAdd->setEnabled(false);
    ui->QPBRemoveProfile->setEnabled(false);
    ui->QRBSideAEqualToSideB->setEnabled(false);
    ui->QRBSideAToSideB->setEnabled(false);
    ui->QRBSideBToSideA->setEnabled(false);
    ui->QTBSideA->setEnabled(false);
    ui->QTBSideB->setEnabled(false);
    QDTextInput textinput;
    textinput.ui->QLTextInput->setText(tr("Profile"));
    textinput.setWindowTitle(tr("New profile"));
    if (textinput.exec()== QDialog::Accepted) {
        Profile profile;
        profile.Description= textinput.ui->QLETextInput->text();
        profile.Direction= SIDEAEQUALTOSDIDEB;
        profile.IgnoreFileDate= false;
        ui->QLWProfiles->addItem(profile.Description);
        QVProfiles.append(profile);
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QPBAnalyze_clicked() {
    if (!ui->QPBAnalyze->isChecked()) {
        delete ThAnalyzeA;
        delete ThAnalyzeB;
        ThAnalyzeA= ThAnalyzeB= NULL;
        ui->QPBAddProfile->setEnabled(true);
        ui->QCBDeleteFilesOnTargetSide->setEnabled(true);
        ui->QCBIgnoreFileDate->setEnabled(true);
        ui->QCBShowDetails->setEnabled(true);
        ui->QCBSnapshot->setEnabled(true);
        ui->QLEDescription->setEnabled(true);
        ui->QLESideA->setEnabled(true);
        ui->QLESideB->setEnabled(true);
        ui->QLWProfiles->setEnabled(true);
        ui->QPBRemoveProfile->setEnabled(true);
        ui->QRBSideAEqualToSideB->setEnabled(true);
        ui->QRBSideAToSideB->setEnabled(true);
        ui->QRBSideBToSideA->setEnabled(true);
        ui->QTBSideA->setEnabled(true);
        ui->QTBSideB->setEnabled(true);
        QTIcons.stop();
        ui->QLUpdate->clear();
        QSTITrayIcon->setIcon(this->windowIcon());
    } else {
        QDTStart= QDateTime::currentDateTime();
        ui->QPBAddProfile->setEnabled(false);
        ui->QCBDeleteFilesOnTargetSide->setEnabled(false);
        ui->QCBIgnoreFileDate->setEnabled(false);
        ui->QCBShowDetails->setEnabled(false);
        ui->QCBSnapshot->setEnabled(false);
        ui->QLEDescription->setEnabled(false);
        ui->QLESideA->setEnabled(false);
        ui->QLESideB->setEnabled(false);
        ui->QLWProfiles->setEnabled(false);
        ui->QPBRemoveProfile->setEnabled(false);
        ui->QPBSynchronize->setEnabled(false);
        ui->QRBSideAEqualToSideB->setEnabled(false);
        ui->QRBSideAToSideB->setEnabled(false);
        ui->QRBSideBToSideA->setEnabled(false);
        ui->QTBSideA->setEnabled(false);
        ui->QTBSideB->setEnabled(false);
        SideAComplete= false;
        SideBComplete= false;
        Mode= MODEANALYZE;
        QTIcons.start();
        ui->QLWSideA->clear();
        ui->QLWSideB->clear();
        ui->QLWBeCopiedToSideA->clear();
        ui->QLWBeCopiedToSideB->clear();
        ui->QLWBeDeletedToSideB->clear();
        ui->QLWBeDeletedToSideA->clear();
        QVSideListA.clear();
        QVSideListB.clear();
        ui->QLWLog->clear();
        QVBeCopiedToSideA.clear();
        QVBeCopiedToSideB.clear();
        QVBeDeletedToSideA.clear();
        QVBeDeletedToSideB.clear();
        ui->QLFilesDirectoriesA->setText("");
        ui->QLFilesDirectoriesB->setText("");
        ui->QLFilesDirectoriesBeCopiedToSideA->setText("");
        ui->QLFilesDirectoriesBeDeleteToSideA->setText("");
        ui->QLFilesDirectoriesBeCopiedToSideB->setText("");
        ui->QLWEmpyDirsOnSideA->clear();
        ui->QLWEmpyDirsOnSideB->clear();
        ui->QPBTotalProgress->setValue(0);
        ui->QLBeDeletedToSideB->setText("");
        GetLog(tr("Begin Analyze..."));
        ThAnalyzeA= new QThAnalyze(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA, SIDEA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles);
        connect(ThAnalyzeA, SIGNAL(SendEmptyDirs(int,QString)), this, SLOT(GetEmptyDirs(int,QString)));
        connect(ThAnalyzeA, SIGNAL(SendFiles(int,long,long,QVector<FileItem>*)), this, SLOT(GetFiles(int,long,long,QVector<FileItem>*)), Qt::BlockingQueuedConnection);
        connect(ThAnalyzeA, SIGNAL(SendAnalyzeComplete(int)), this, SLOT(GetAnalyzeComplete(int)));
        ThAnalyzeA->start();
        if (!ui->QCBSnapshot->isChecked()) {
            ThAnalyzeB= new QThAnalyze(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB, SIDEB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles);
            connect(ThAnalyzeB, SIGNAL(SendEmptyDirs(int,QString)), this, SLOT(GetEmptyDirs(int,QString)));
            connect(ThAnalyzeB, SIGNAL(SendFiles(int,long,long,QVector<FileItem>*)), this, SLOT(GetFiles(int,long,long,QVector<FileItem>*)), Qt::BlockingQueuedConnection);
            connect(ThAnalyzeB, SIGNAL(SendAnalyzeComplete(int)), this, SLOT(GetAnalyzeComplete(int)));
            ThAnalyzeB->start();
        } else SideBComplete= true;
    }
}

void QFMainWindow::on_QPBHide_clicked() {
    this->hide();
}

void QFMainWindow::on_QPBEmptyDirsSideADelete_clicked() {
    bool Trovato= false;
    for (int count= ui->QLWEmpyDirsOnSideA->count()- 1; count> -1; count--) {
        if (ui->QLWEmpyDirsOnSideA->item(count)->isSelected()) {
            Trovato= true;
            break;
        }
    }
    if (Trovato) {
        switch (QMessageBox(tr("Warning!"), tr("Are you sure?"), QMessageBox::Warning, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Escape).exec()) {
        case 3: {
                for (int count= ui->QLWEmpyDirsOnSideA->count()- 1; count> -1; count--) {
                    if (ui->QLWEmpyDirsOnSideA->item(count)->isSelected()) {
                        QDir Dir;
                        if (Dir.rmdir(ui->QLWEmpyDirsOnSideA->item(count)->text())) {
                            GetLog(ui->QLWEmpyDirsOnSideA->item(count)->text()+ tr(" deleted"));
                            ui->QLWEmpyDirsOnSideA->takeItem(ui->QLWEmpyDirsOnSideA->row(ui->QLWEmpyDirsOnSideA->item(count)));
                        } else GetLog(ui->QLWEmpyDirsOnSideA->item(count)->text()+ tr(" error on delete!"), "#ff0000");
                    }
                }
                break;
            }
        }
    } else QMessageBox::critical(this, tr("Error!!!"), tr("No files selected!"), "Ok");
}

void QFMainWindow::on_QPBEmptyDirsSideASelectAll_clicked() {
    ui->QLWEmpyDirsOnSideA->selectAll();
}

void QFMainWindow::on_QPBEmptyDirsSideASelectNone_clicked() {
    for (int count= ui->QLWEmpyDirsOnSideA->count()- 1; count> -1; count--) ui->QLWEmpyDirsOnSideA->item(count)->setSelected(false);
}

void QFMainWindow::on_QPBEmptyDirsSideBDelete_clicked() {
    bool Trovato= false;
    for (int count= ui->QLWEmpyDirsOnSideB->count()- 1; count> -1; count--) {
        if (ui->QLWEmpyDirsOnSideB->item(count)->isSelected()) {
            Trovato= true;
            break;
        }
    }
    if (Trovato) {
        switch (QMessageBox(tr("Warning!"), tr("Are you sure?"), QMessageBox::Warning, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Escape).exec()) {
        case 3: {
                for (int count= ui->QLWEmpyDirsOnSideB->count()- 1; count> -1; count--) {
                    if (ui->QLWEmpyDirsOnSideB->item(count)->isSelected()) {
                        QDir Dir;
                        if (Dir.rmdir(ui->QLWEmpyDirsOnSideB->item(count)->text())) {
                            GetLog(ui->QLWEmpyDirsOnSideB->item(count)->text()+ " deleted");
                            ui->QLWEmpyDirsOnSideB->takeItem(ui->QLWEmpyDirsOnSideB->row(ui->QLWEmpyDirsOnSideB->item(count)));
                        } else GetLog(ui->QLWEmpyDirsOnSideB->item(count)->text()+ " error on delete!", "#ff0000");
                    }
                }
                break;
            }
        }
    } else QMessageBox::critical(this, tr("Error!!!"), tr("No files selected!"), "Ok");
}

void QFMainWindow::on_QPBEmptyDirsSideBSelectAll_clicked() {
    ui->QLWEmpyDirsOnSideB->selectAll();
}

void QFMainWindow::on_QPBEmptyDirsSideBSelectNone_clicked() {
    for (int count= ui->QLWEmpyDirsOnSideB->count()- 1; count> -1; count--) ui->QLWEmpyDirsOnSideB->item(count)->setSelected(false);
}

void QFMainWindow::on_QPBExclusionListRemove_clicked() {
    switch (QMessageBox(tr("Warning!"), tr("Are you sure?"), QMessageBox::Warning, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Escape).exec()) {
    case 3: {
            ui->QPBExclusionListRemove->setEnabled(false);
            QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles.removeAt(ui->QLWExclusionList->row(ui->QLWExclusionList->currentItem()));
            ui->QLWExclusionList->clear();
            ui->QLWExclusionList->addItems(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].QSLExclusionFiles);
            ui->QPBSave->setEnabled(true);
            break;
        }
    }
}

void QFMainWindow::on_QPBQuit_clicked() {
    if (ui->QPBSave->isEnabled()) {
        switch (QMessageBox(tr("Warning!"), tr("Are you sure to exit without save?"), QMessageBox::Warning, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Escape).exec()) {
        case 3: {
                delete QSTITrayIcon;
                this->close();
                break;
            }
        }
    } else {
        delete QSTITrayIcon;
        this->close();
    }
}

void QFMainWindow::on_QPBRemoveProfile_clicked() {
    switch (QMessageBox(tr("Warning!"), tr("Are you sure?"), QMessageBox::Warning, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Escape).exec()) {
    case 3: {
            ui->QLEDescription->setEnabled(false);
            ui->QCBDeleteFilesOnTargetSide->setEnabled(false);
            ui->QCBIgnoreFileDate->setEnabled(false);
            ui->QCBShowDetails->setEnabled(false);
            ui->QCBSnapshot->setEnabled(false);
            ui->QLESideA->setEnabled(false);
            ui->QLESideB->setEnabled(false);
            ui->QLWExclusionList->setEnabled(false);
            ui->QPBRemoveProfile->setEnabled(false);
            ui->QRBSideAEqualToSideB->setEnabled(false);
            ui->QRBSideAToSideB->setEnabled(false);
            ui->QRBSideBToSideA->setEnabled(false);
            ui->QTBExclusionListAdd->setEnabled(false);
            ui->QTBSideA->setEnabled(false);
            ui->QTBSideB->setEnabled(false);
            QVProfiles.remove(ui->QLWProfiles->row(ui->QLWProfiles->currentItem()));
            ui->QLWProfiles->takeItem(ui->QLWProfiles->row(ui->QLWProfiles->currentItem()));
            ui->QPBSave->setEnabled(true);
            break;
        }
    }
}

void QFMainWindow::on_QPBSave_clicked() {
    QSettings *QSSettings= new QSettings(QDir::homePath()+ QDir::separator()+ ".BackupTo"+ QDir::separator()+ "BackupTo.ini", QSettings::IniFormat); {
        QSSettings->beginGroup("Profiles");
        QSSettings->remove("");
        for (int count= 0; count< QVProfiles.count(); count++) {
            QSSettings->setValue("Description"+ QString::number(count), QVProfiles.at(count).Description);
            QSSettings->setValue("Direction"+ QString::number(count), QVProfiles.at(count).Direction);
            QSSettings->setValue("IgnoreFileDate"+ QString::number(count), QVProfiles.at(count).IgnoreFileDate);
            QSSettings->setValue("SideA"+ QString::number(count), QVProfiles.at(count).SideA);
            QSSettings->setValue("SideB"+ QString::number(count), QVProfiles.at(count).SideB);
            QSSettings->setValue("ShowDetails"+ QString::number(count), QVProfiles.at(count).ShowDetails);
            QSSettings->setValue("Snapshot"+ QString::number(count), QVProfiles.at(count).Snapshot);
            QSSettings->setValue("DeleteFilesOnTargetSide"+ QString::number(count), QVProfiles.at(count).DeleteFilesOnTargetSide);
            for (int count_2= 0; count_2< QVProfiles.at(count).QSLExclusionFiles.size(); count_2++) {
                QSSettings->setValue("ExclusionList"+ QString::number(count)+ "."+ QString::number(count_2), QVProfiles.at(count).QSLExclusionFiles.at(count_2));
            }
        }
        QSSettings->endGroup();
    }{
        delete QSSettings;
    }
    ui->QPBSave->setEnabled(false);
}

void QFMainWindow::on_QPBStop_clicked() {
    QTIcons.stop();
}

void QFMainWindow::on_QPBSynchronize_clicked() {
    if (!ui->QPBSynchronize->isChecked()) {
        delete ThSynchronize;
        ThSynchronize= NULL;
        QTIcons.stop();
        ui->QLUpdate->clear();
        QSTITrayIcon->setIcon(this->windowIcon());
        ui->QPBTotalProgress->setValue(ui->QPBTotalProgress->maximum());
        Mode= MODENULL;
    } else {
        QDTStart= QDateTime::currentDateTime();
        SideAComplete= false;
        Mode= MODESYNCHRONYZE;
        if (ui->QCBSnapshot->isChecked()) {
            GetLog(tr("Begin Snapshot..."));
            ui->QPBTotalProgress->setMaximum(QVBeCopiedToSideB.size());
            QDir Dir;
            Dir.mkdir(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QDir::separator()+ QDateTime::currentDateTime().toString("yyyyMMdd"));
            ThSynchronize= new QThSynchronize(this, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QDir::separator()+ QDateTime::currentDateTime().toString("yyyyMMdd"));
        } else {
            GetLog(tr("Begin Synchronization..."));
            ui->QPBTotalProgress->setMaximum(QVBeCopiedToSideA.size()+ QVBeCopiedToSideB.size()+ QVBeDeletedToSideA.size()+ QVBeDeletedToSideB.size());
            ThSynchronize= new QThSynchronize(this, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB);
        }
        ui->QPBTotalProgress->setValue(0);
        connect(ThSynchronize, SIGNAL(GetSynchronizeComplete()), this, SLOT(GetSynchronizeComplete()));
        connect(ThSynchronize, SIGNAL(SendLog(QString,QString,long)), this, SLOT(GetLog(QString,QString,long)), Qt::BlockingQueuedConnection);
        ThSynchronize->start();
        QTIcons.start();
    }
}

void QFMainWindow::on_QRBSideAEqualToSideB_toggled(bool) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Direction= SIDEAEQUALTOSDIDEB;
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Snapshot= false;
        ui->QCBSnapshot->setChecked(false);
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QRBSideAToSideB_toggled(bool) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Direction= SIDEATOSIDEB;
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QRBSideBToSideA_toggled(bool) {
    if (!UpdateInAction) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Direction= SIDEBTOSDIDEA;
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].Snapshot= false;
        ui->QCBSnapshot->setChecked(false);
        ui->QPBSave->setEnabled(true);
    }
}

void QFMainWindow::on_QTBSideA_clicked() {
    UpdateInAction= true;
    QString Path= QFileDialog::getExistingDirectory(this, "Side A", QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA, QFileDialog::ShowDirsOnly);
    if (Path!= NULL) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA= Path;
        ui->QLESideA->setText(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA);
        ui->QPBSave->setEnabled(true);
    }
    UpdateInAction= false;
}

void QFMainWindow::on_QTBSideB_clicked() {
    UpdateInAction= true;
    QString Path= QFileDialog::getExistingDirectory(this, "Side B", QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB, QFileDialog::ShowDirsOnly);
    if (Path!= NULL) {
        QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB= Path;
        ui->QLESideB->setText(QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB);
        ui->QPBSave->setEnabled(true);
    }
    UpdateInAction= false;
}

void QFMainWindow::SideList(QString Path) {
    QDir QDPath= QDir(Path);
    QStringList QSLFiles= QDPath.entryList(QDir::AllEntries | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    for (int count= 0; count< QSLFiles.size(); count++) {
        ui->QLWSideA->addItem(Path+ QDir::separator()+ QSLFiles[count]);
        QFileInfo QFIPath(Path+ QDir::separator()+ QSLFiles[count]);
        if (QFIPath.isDir()) SideList(QFIPath.absoluteFilePath());
    }
}

void QFMainWindow::UpdateIcons() {
    switch (LastIcon) {
        case 0: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/00.png"))); break;
        case 1: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/01.png"))); break;
        case 2: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/02.png"))); break;
        case 3: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/03.png"))); break;
        case 4: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/04.png"))); break;
        case 5: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/05.png"))); break;
        case 6: ui->QLUpdate->setPixmap(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/06.png"))); break;
    }
    switch (LastIcon) {
        case 0: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/00.png")))); break;
        case 1: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/01.png")))); break;
        case 2: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/02.png")))); break;
        case 3: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/03.png")))); break;
        case 4: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/04.png")))); break;
        case 5: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/05.png")))); break;
        case 6: QSTITrayIcon->setIcon(QIcon(QPixmap(QString::fromUtf8(":/IconeRefresh/IconeRefresh/06.png")))); break;
    }

    LastIcon++;
    if (LastIcon> 6) LastIcon= 0;
    if (Mode== MODEANALYZE && SideAComplete && SideBComplete) {
        QTIcons.stop();
        ui->QLUpdate->clear();
        QSTITrayIcon->setIcon(this->windowIcon());
        delete ThAnalyzeA;
        delete ThAnalyzeB;
        ThAnalyzeA= ThAnalyzeB= NULL;
        ui->QPBAnalyze->setChecked(false);
        long Directory= 0;
        long Files= 0;
        if (!ui->QCBSnapshot->isChecked()) {
            if (ui->QRBSideAToSideB->isChecked()) {
                for (int count= QVSideListA.size()- 1; count> -1; count--) {
                    bool Trovato= false;
                    QString SideA= QVSideListA[count].Path+ QVSideListA[count].File;
                    #ifdef Q_WS_WIN
                        SideA= SideA.toLower();
                    #endif
                    for (int count_2= QVSideListB.size()- 1; count_2> -1; count_2--) {
                        QString SideB= QVSideListB[count_2].Path+ QVSideListB[count_2].File;
                        #ifdef Q_WS_WIN
                            SideB= SideB.toLower();
                        #endif
                        if (SideA.compare(SideB)== 0 && QVSideListA[count].IsDirectory== QVSideListB[count_2].IsDirectory && (QVSideListA[count].IsDirectory || (!QVSideListA[count].IsDirectory && QVSideListA[count].Size== QVSideListB[count_2].Size && (ui->QCBIgnoreFileDate->isChecked() || (SecondsBetween(QVSideListA[count].LastModified, QVSideListB[count_2].LastModified)< 2))))) {
                            Trovato= true;
                            QVSideListA.remove(count, 1);
                            QVSideListB.remove(count_2, 1);
                            break;
                        } else if (SideA.compare(SideB)== 0 && QVSideListA[count].IsDirectory== QVSideListB[count_2].IsDirectory) {
                            QVSideListB.remove(count_2, 1);
                            break;
                        }
                        if (SideA.compare(SideB, Qt::CaseSensitive)> 0) break;
                    }
                    if (!Trovato) {
                        QVBeCopiedToSideB.append(QVSideListA[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::blue);
                                Files++;
                            }
                        }
                        QVSideListA.remove(count, 1);
                    }
                }
                ui->QLFilesDirectoriesBeCopiedToSideB->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
                if (QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].DeleteFilesOnTargetSide) {
                    Directory= 0;
                    Files= 0;
                    for (int count= QVSideListB.size()- 1; count> -1; count--) {
                        QVBeDeletedToSideB.append(QVSideListB[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeDeletedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeDeletedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeDeletedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::blue);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeDeletedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::blue);
                                Files++;
                            }
                        }
                    }
                    ui->QLBeDeletedToSideB->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
                }
            } else if (ui->QRBSideAEqualToSideB->isChecked()) {
                for (int count= QVSideListA.size()- 1; count> -1; count--) {
                    QString SideA= QVSideListA[count].Path+ QVSideListA[count].File;
                    #ifdef Q_WS_WIN
                        SideA= SideA.toLower();
                    #endif
                    bool Trovato= false;
                    for (int count_2= QVSideListB.size()- 1; count_2> -1; count_2--) {
                        QString SideB= QVSideListB[count_2].Path+ QVSideListB[count_2].File;
                        #ifdef Q_WS_WIN
                            SideB= SideB.toLower();
                        #endif
                        if (SideA.compare(SideB)== 0 && QVSideListA[count].IsDirectory== QVSideListB[count_2].IsDirectory && (QVSideListA[count].IsDirectory || (!QVSideListA[count].IsDirectory && (ui->QCBIgnoreFileDate->isChecked() || (QVSideListA[count].LastModified<= QVSideListB[count_2].LastModified))))) {
                            Trovato= true;
                            QVSideListA.remove(count, 1);
                            QVSideListB.remove(count_2, 1);
                            break;
                        }
                    }
                    if (!Trovato) {
                        QVBeCopiedToSideB.append(QVSideListA[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::blue);
                                Files++;
                            }
                        }
                        QVSideListA.remove(count, 1);
                    }
                }
                ui->QLFilesDirectoriesBeCopiedToSideB->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
                Directory= 0;
                Files= 0;
                for (int count= QVSideListB.size()- 1; count> -1; count--) {
                    QString SideB= QVSideListB[count].Path+ QVSideListB[count].File;
                    #ifdef Q_WS_WIN
                        SideB= SideB.toLower();
                    #endif
                    bool Trovato= false;
                    for (int count_2= QVSideListA.size()- 1; count_2> -1; count_2--) {
                        QString SideA= QVSideListA[count_2].Path+ QVSideListA[count_2].File;
                        #ifdef Q_WS_WIN
                            SideA= SideA.toLower();
                        #endif
                        if (SideB.compare(SideA)== 0 && QVSideListB[count].IsDirectory== QVSideListA[count_2].IsDirectory/* && (QVSideListB[count].IsDirectory || (!QVSideListB[count].IsDirectory && (ui->QCBIgnoreFileDate->isChecked() || (QVSideListB[count].LastModified<= QVSideListA[count_2].LastModified))))*/) {
                            Trovato= true;
                            QVSideListA.remove(count_2, 1);
                            QVSideListB.remove(count, 1);
                            break;
                        }
                    }
                    if (!Trovato) {
                        QVBeCopiedToSideA.append(QVSideListB[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::blue);
                                Files++;
                            }
                        }
                        QVSideListB.remove(count, 1);
                    }
                }
                ui->QLFilesDirectoriesBeCopiedToSideA->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
            } else if (ui->QRBSideBToSideA) {
                for (int count= QVSideListB.size()- 1; count> -1; count--) {
                    bool Trovato= false;
                    QString SideB= QVSideListB[count].Path+ QVSideListB[count].File;
                    #ifdef Q_WS_WIN
                        SideB= SideB.toLower();
                    #endif
                    for (int count_2= QVSideListA.size()- 1; count_2> -1; count_2--) {
                        QString SideA= QVSideListA[count_2].Path+ QVSideListA[count_2].File;
                        #ifdef Q_WS_WIN
                            SideA= SideA.toLower();
                        #endif
                        if (SideB.compare(SideA)== 0 && QVSideListB[count].IsDirectory== QVSideListA[count_2].IsDirectory && (QVSideListB[count].IsDirectory || (!QVSideListB[count].IsDirectory && QVSideListB[count].Size== QVSideListA[count_2].Size && (ui->QCBIgnoreFileDate->isChecked() || (SecondsBetween(QVSideListB[count].LastModified, QVSideListA[count_2].LastModified)< 2))))) {
                            Trovato= true;
                            QVSideListB.remove(count, 1);
                            QVSideListA.remove(count_2, 1);
                            break;
                        } else if (SideB.compare(SideA)== 0 && QVSideListB[count].IsDirectory== QVSideListA[count_2].IsDirectory) {
                            QVSideListA.remove(count_2, 1);
                            break;
                        }
                    }
                    if (!Trovato) {
                        QVBeCopiedToSideA.append(QVSideListB[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File+ " FileSize: "+ QString::number(QVSideListB[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListB[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListB[count].IsDirectory) {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeCopiedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideB+ QVSideListB[count].Path+ QVSideListB[count].File, Qt::blue);
                                Files++;
                            }
                        }
                        QVSideListB.remove(count, 1);
                    }
                }
                ui->QLFilesDirectoriesBeCopiedToSideA->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
                if (QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].DeleteFilesOnTargetSide) {
                    Directory= 0;
                    Files= 0;
                    for (int count= QVSideListA.size()- 1; count> -1; count--) {
                        QVBeDeletedToSideA.append(QVSideListA[count]);
                        if (ui->QCBShowDetails->isChecked()) {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeDeletedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeDeletedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                                Files++;
                            }
                        } else {
                            if (QVSideListA[count].IsDirectory) {
                                AddItem(ui->QLWBeDeletedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::black);
                                Directory++;
                            } else {
                                AddItem(ui->QLWBeDeletedToSideA, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::blue);
                                Files++;
                            }
                        }
                    }
                    ui->QLFilesDirectoriesBeDeleteToSideA->setText(QString::number(Files)+ " files, "+ QString::number(Directory)+ " directories.");
                }
            }
        } else {
            for (int count= QVSideListA.size()- 1; count> -1; count--) {
                QVBeCopiedToSideB.append(QVSideListA[count]);
                if (ui->QCBShowDetails->isChecked()) {
                    if (QVSideListA[count].IsDirectory) {
                        AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::black);
                        Directory++;
                    } else {
                        AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File+ " FileSize: "+ QString::number(QVSideListA[count].Size)+ ", Last modified: "+ QDateTime::fromTime_t(QVSideListA[count].LastModified).toString("yyyy-MM-dd HH:mm:ss"), Qt::blue);
                        Files++;
                    }
                } else {
                    if (QVSideListA[count].IsDirectory) {
                        AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::black);
                        Directory++;
                    } else {
                        AddItem(ui->QLWBeCopiedToSideB, QVProfiles[ui->QLWProfiles->row(ui->QLWProfiles->currentItem())].SideA+ QVSideListA[count].Path+ QVSideListA[count].File, Qt::blue);
                        Files++;
                    }
                }
            }
            QVBeCopiedToSideB= QVSideListA;
        }
        ui->QLWProfiles->setEnabled(true);
        ui->QPBAddProfile->setEnabled(true);
        ui->QPBSynchronize->setEnabled(true);
    } else if (Mode== MODESYNCHRONYZE && SideAComplete) {
        QTIcons.stop();
        ui->QLUpdate->clear();
        QSTITrayIcon->setIcon(this->windowIcon());
        delete ThSynchronize;
        ThSynchronize= NULL;
        ui->QPBSynchronize->setChecked(false);
        ui->QPBTotalProgress->setValue(ui->QPBTotalProgress->maximum());
    }
}
