/***************************************************************************
*   Copyright (C) 2011-2017 by Denis Gottardello                          *
*   info@denisgottardello.it                                              *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 3 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef QFMAINWINDOW_H
#define QFMAINWINDOW_H

#include <QMainWindow>
#include "QSystemTrayIcon"
#include "QSettings"
#include "QDir"
#include "commons.h"
#include "QVector"
#include "qdtextinput.h"
#include "QMessageBox"
#include "QFileDialog"
#include "QTimer"
#include "QDebug"
#include "qthanalyze.h"
#include "QMenu"
#include "QDateTime"
#ifndef THSYNCHRONIZE_H
    class QFMainWindow;
    #include "qthsynchronize.h"
#endif
#include "QDesktopServices"
#include "QUrl"
#include "qdfileattributes.h"
#include "QMenu"
#include "qdemptydirectory.h"
#include "qdhtmlpopup.h"
#include <QListWidget>

namespace Ui {
class QFMainWindow;
}

class QFMainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit QFMainWindow(QWidget *parent = 0);
    ~QFMainWindow();
    int Mode;
    QVector<FileItem> QVBeCopiedToSideA, QVBeCopiedToSideB, QVBeDeletedToSideA, QVBeDeletedToSideB;
    QVector<Profile> QVProfiles;

private slots:
    void AddFiles();
    void AddDirectories();
    void EmptyDirectory();
    void EmptyDirsAPopup(QPoint pos);
    void EmptyDirsBPopup(QPoint pos);
    void FileAttributes();
    void GetAnalyzeComplete(int Side);
    void GetEmptyDirs(int Side, QString Path);
    void GetFiles(int Side, long Files, long Directories, QVector<FileItem> *QVSideList);
    void GetLog(QString Row);
    void GetLog(QString Row, QString Color);
    void GetLog(QString Row, QString Color, long Objects);
    void GetSynchronizeComplete();
    void on_QAAuthor_triggered();
    void on_QAQtVersion_triggered();
    void on_QAVersion_triggered();
    void on_QCBDeleteFilesOnTargetSide_toggled(bool checked);
    void on_QCBIgnoreFileDate_toggled(bool checked);
    void on_QCBShowDetails_toggled(bool checked);
    void on_QCBSnapshot_toggled(bool checked);
    void on_QLEDescription_textChanged(const QString &arg1);
    void on_QLESideA_textChanged(const QString &arg1);
    void on_QLESideB_textChanged(const QString &arg1);
    void on_QLWExclusionList_currentRowChanged(int currentRow);
    void on_QLWProfiles_clicked(const QModelIndex &index);
    void on_QLWProfiles_currentRowChanged(int currentRow);
    void on_QPBAddProfile_clicked();
    void on_QPBAnalyze_clicked();
    void on_QPBEmptyDirsSideADelete_clicked();
    void on_QPBEmptyDirsSideASelectAll_clicked();
    void on_QPBEmptyDirsSideASelectNone_clicked();
    void on_QPBEmptyDirsSideBDelete_clicked();
    void on_QPBEmptyDirsSideBSelectAll_clicked();
    void on_QPBEmptyDirsSideBSelectNone_clicked();
    void on_QPBExclusionListRemove_clicked();
    void on_QPBHide_clicked();
    void on_QPBQuit_clicked();
    void on_QPBRemoveProfile_clicked();
    void on_QPBSave_clicked();
    void on_QPBStop_clicked();
    void on_QPBSynchronize_clicked();
    void on_QRBSideAEqualToSideB_toggled(bool checked);
    void on_QRBSideAToSideB_toggled(bool checked);
    void on_QRBSideBToSideA_toggled(bool checked);
    void on_QTBSideA_clicked();
    void on_QTBSideB_clicked();
    void UpdateIcons();

private:
    Ui::QFMainWindow *ui;
    bool UpdateInAction, SideAComplete, SideBComplete;
    int LastIcon;
    long Objects;
    QDateTime QDTStart;
    QDHTMLPOpup *HTMLPOpup;
    QMenu *QMEmptyDirsA, *QMEmptyDirsB;
    QSystemTrayIcon *QSTITrayIcon;
    QThAnalyze *ThAnalyzeA, *ThAnalyzeB;
    QThSynchronize *ThSynchronize;
    QTimer QTIcons;
    QVector<FileItem> QVSideListA, QVSideListB;
    void AddItem(QListWidget *QLWList, QString Path, QBrush Color);
    void SideList(QString Path);

};

#endif // QFMAINWINDOW_H
