/***************************************************************************
*   Copyright (C) 2011-2017 by Denis Gottardello                          *
*   info@denisgottardello.it                                              *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 3 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "qthanalyze.h"

QThAnalyze::QThAnalyze(QString InitialPath, int Side, QStringList QSLExclusionFiles) : QThread() {
    this->InitialPath= InitialPath;
    this->Side= Side;
    for (int count= 0; count< QSLExclusionFiles.size(); count++) QVExclusionFiles.append(QFileInfo(QSLExclusionFiles[count]));
    DoStart= true;
}

QThAnalyze::~QThAnalyze() {
    DoStart= false;
    while (this->isRunning()) msleep(10);
}

void QThAnalyze::run() {
    Files= 0;
    Directories= 0;
    SideList(InitialPath);
    //sleep(10);
    if (DoStart) emit SendAnalyzeComplete(Side);
}

void QThAnalyze::SideList(QString Path) {
    QVector<FileItem> *QVSideList= new QVector<FileItem>();
    QDir QDPath= QDir(Path);
    QDPath.setSorting(QDir::Name);
    QStringList QSLFiles= QDPath.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::Hidden);
    if (QSLFiles.size()== 0) emit SendEmptyDirs(Side, Path);
    else {
        for (int count= 0; count< QSLFiles.size(); count++) {
            FileItem fileitem;
            fileitem.Path= Path+ QDir::separator();
            if (fileitem.Path.startsWith(InitialPath)) fileitem.Path= fileitem.Path.right(fileitem.Path.length()- InitialPath.length());
            fileitem.File= QSLFiles[count];
            QFileInfo QFIPath(Path+ QDir::separator()+ QSLFiles[count]);
            struct stat FileAttributes;
            stat(QString(Path+ QDir::separator()+ QSLFiles[count]).toStdString().c_str(), &FileAttributes);
            if (QFIPath.isDir()) {
                bool ToBeExcluded= false;
                for (int count_2= 0; count_2< QVExclusionFiles.size(); count_2++) {
                    if (QVExclusionFiles[count_2].isDir() && QVExclusionFiles[count_2].filePath().compare(QFIPath.filePath())== 0) {
                        ToBeExcluded= true;
                        break;
                    }
                }
                if (!ToBeExcluded) {
                    fileitem.IsDirectory= true;
                    Directories++;
                    QVSideList->append(fileitem);
                    if (DoStart) emit SendFiles(Side, Files, Directories, QVSideList);
                    QVSideList->clear();
                    if (DoStart && QFIPath.isDir() &! QFIPath.isSymLink() &! QFIPath.isBundle()) SideList(Path+ QDir::separator()+ QSLFiles[count]);
                }
            } else {
                bool ToBeExcluded= false;
                for (int count_2= 0; count_2< QVExclusionFiles.size(); count_2++) {
                    if (QVExclusionFiles[count_2].isFile() && QVExclusionFiles[count_2].filePath().compare(QFIPath.filePath())== 0) {
                        ToBeExcluded= true;
                        break;
                    }
                }
                if (!ToBeExcluded) {
                    fileitem.IsDirectory= false;
                    fileitem.Size= QFIPath.size();
                    Files++;
                    fileitem.LastAccess= FileAttributes.st_atime;
                    fileitem.LastModified= FileAttributes.st_mtime;
                    QVSideList->append(fileitem);
                }
            }
        }
        if (DoStart) emit SendFiles(Side, Files, Directories, QVSideList);
    }
}
